import { z } from 'zod';
import { referenceSchema } from './referenceSchema';
import { glyphSchema } from './glyphSchema';
import { modificationResiduesSchema } from './modificationResiduesSchema';
import { submodelSchema } from './submodelSchema';
import { colorSchema } from './colorSchema';
import { productsSchema } from './products';
import { lineSchema } from './lineSchema';
import { operatorSchema } from './operatorSchema';

export const bioEntitySchema = z.object({
  id: z.number(),
  stringType: z.string(),
  name: z.string(),
  elementId: z.string(),
  model: z.number(),
  references: z.array(referenceSchema),
  z: z.number(),
  notes: z.string(),
  symbol: z.string(),
  homodimer: z.number(),
  nameX: z.number(),
  nameY: z.number(),
  nameWidth: z.number(),
  nameHeight: z.number(),
  nameVerticalAlign: z.string(),
  nameHorizontalAlign: z.string(),
  width: z.number(),
  height: z.number(),
  visibilityLevel: z.string(),
  transparencyLevel: z.string(),
  synonyms: z.array(z.string()),
  formerSymbols: z.array(z.string()),
  fullName: z.string(),
  abbreviation: z.string(),
  formula: z.string(),
  glyph: z.union([glyphSchema, z.null()]),
  activity: z.boolean(),
  structuralState: z.string(),
  hypothetical: z.boolean(),
  boundaryCondition: z.boolean(),
  constant: z.boolean(),
  initialAmount: z.number(),
  initialConcentration: z.number(),
  charge: z.number(),
  substanceUnits: z.string(),
  onlySubstanceUnits: z.boolean(),
  modificationResidues: modificationResiduesSchema,
  complex: z.number(),
  compartment: z.number(),
  submodel: submodelSchema,
  x: z.number(),
  y: z.number(),
  lineWidth: z.number(),
  fontColor: colorSchema,
  fontSize: z.number(),
  fillColor: colorSchema,
  borderColor: colorSchema,
  smiles: z.string(),
  inChI: z.string(),
  inChIKey: z.string(),
  thickness: z.number(),
  outerWidth: z.number(),
  innerWidth: z.number(),
  idReaction: z.string(),
  reversible: z.boolean(),
  mechanicalConfidenceScore: z.boolean(),
  lowerBound: z.boolean(),
  upperBound: z.boolean(),
  subsystem: z.string(),
  geneProteinReaction: z.string(),
  kinetics: z.union([z.null(), z.undefined()]),
  products: z.union([z.array(productsSchema), z.undefined()]),
  reactants: z.union([z.array(productsSchema), z.undefined()]),
  modifiers: z.union([z.array(productsSchema), z.undefined()]),
  processCoordinates: z.union([z.null(), z.undefined()]),
  line: z.union([lineSchema, z.undefined()]),
  operators: z.union([z.array(operatorSchema), z.undefined()]),
});
